# Anniina Jaakkonen, 16 Nov 2018

# Risk of milk contamination by stx (STEC)
#########################################################

# R file

# y[i]: number of stx-positive milk samples in sampling i
# n[i]: number of milk samples in sampling i
# th[i]: proportion of stx-positive milk samples in sampling i

# Ns (max. 157): number of samplings (observations)
# Nf (3): number of farms (levels of the factor Farm)
# Nv (max. 9): number of variables in the model

#######################################
# model with 9 explanatory variables and fixed effect Farm

library(rjags)

# setwd("path/to/input/directory")

# load and put the data into a list format

data <- read.table ("milk_contamination_data.txt", header=TRUE, dec=".")

str(data)

# response variable:
y <- data$MilkPosStx
n <- data$MilkNPCR

# explanatory variables:
Farm <- as.factor(data$Farm)
Sale <- data$SaleCow
Cleanse <- data$Cleanse
Feed <- data$Feed
Pastured <- data$PasturedCow
Maint <- data$MaintMilkOther
Rain <- data$RaiNAays
Temp <- (data$AvTemp - mean(data$AvTemp))/sd(data$AvTemp)
Bact <- (data$BactClosest - mean(data$BactClosest, na.rm = TRUE))/sd(data$BactClosest, na.rm = TRUE)
Cell <- (data$CellClosest - mean(data$CellClosest, na.rm = TRUE))/sd(data$CellClosest, na.rm = TRUE)

Ns <- nrow(data)
Nf <- as.integer(3)
Nv <- as.integer(9)

data_list <- list ("y" = y, "n" = n, 
                    "Farm" = Farm, "Sale" = Sale,
                    "Cleanse" = Cleanse, "Feed" = Feed,
                    "Pastured" = Pastured, "Maint" = Maint,
                    "Rain" = Rain, "Temp" = Temp,
                    "Bact" = Bact, "Cell" = Cell,
                    "Ns"= Ns, "Nf" = Nf, "Nv" = Nv)

# define sampling options and 
# run the Markov chain sampling with JAGS:

monitor = c("beta0", "beta")

jm<-jags.model("milk_contamination_model.txt",data=data_list,n.chains=2,n.adapt=200)
chains<-coda.samples(jm,monitor,n.iter=10000,thin=2)

# check convergence
plot(chains)
gelman.diag(chains)

# check autocorrelation
chain1 <- as.matrix(chains[1])
chain2 <- as.matrix(chains[2])

acf(chain1[,1:4])
acf(chain1[,5:8])
acf(chain1[,9:12])

acf(chain2[,1:4])
acf(chain2[,5:8])
acf(chain2[,9:12])

# combine chains
chains_combined <- as.matrix(chains)

# rename posteriors of beta0 and beta
# by including factor and variable names
beta0_Farm1 <- chains_combined[,"beta0[1]"]
beta0_Farm2 <- chains_combined[,"beta0[2]"]
beta0_Farm3 <- chains_combined[,"beta0[3]"]

beta_Sale <- chains_combined[,"beta[1]"]
beta_Cleanse <- chains_combined[,"beta[2]"]
beta_Feed <- chains_combined[,"beta[3]"]
beta_Pastured <- chains_combined[,"beta[4]"]
beta_Maint <- chains_combined[,"beta[5]"]
beta_Rain <- chains_combined[,"beta[6]"]
beta_Temp <- chains_combined[,"beta[7]"]
beta_Bact <- chains_combined[,"beta[8]"]
beta_Cell <- chains_combined[,"beta[9]"]

# draw boxplots for the posterior of beta0s
par(mfrow=c(1,1))
boxplot(beta0_Farm1,beta0_Farm2, beta0_Farm3, 
        names=c("Farm 1", "Farm 2", "Farm 3"),
        main="Posterior of beta0")

# farm effect can be seen from the boxplots

# calculate 95% posterior credibility intervals for betas
q95_beta_Sale <- quantile(beta_Sale, probs = c(0.05, 0.95))
q95_beta_Cleanse <- quantile(beta_Cleanse, probs = c(0.05, 0.95))
q95_beta_Feed <- quantile(beta_Feed, probs = c(0.05, 0.95))
q95_beta_Pastured <- quantile(beta_Pastured, probs = c(0.05, 0.95))
q95_beta_Maint <- quantile(beta_Maint, probs = c(0.05, 0.95))
q95_beta_Rain <- quantile(beta_Rain, probs = c(0.05, 0.95))
q95_beta_Temp <- quantile(beta_Temp, probs = c(0.05, 0.95))
q95_beta_Bact <- quantile(beta_Bact, probs = c(0.05, 0.95))
q95_beta_Cell <- quantile(beta_Cell, probs = c(0.05, 0.95))

# draw densityplots for the posterior of betas
par(mfrow=c(3,3))

plot(density(beta_Sale), main="Posterior of beta: Sale")
  abline(v=q95_beta_Sale, col="red")
  
plot(density(beta_Cleanse), main="Posterior of beta: Cleanse")
  abline(v=q95_beta_Cleanse, col="red")
  
plot(density(beta_Feed), main="Posterior of beta: Feed")
  abline(v=q95_beta_Feed, col="red")

plot(density(beta_Pastured), main="Posterior of beta: Pastured")
  abline(v=q95_beta_Pastured, col="red")

plot(density(beta_Maint), main="Posterior of beta: Maint")
  abline(v=q95_beta_Maint, col="red")

plot(density(beta_Rain), main="Posterior of beta: Rain")
  abline(v=q95_beta_Rain, col="red")

plot(density(beta_Temp), main="Posterior of beta: Temp")
  abline(v=q95_beta_Temp, col="red")

plot(density(beta_Bact), main="Posterior of beta: Bact")
  abline(v=q95_beta_Bact, col="red")

plot(density(beta_Cell), main="Posterior of beta: Cell")
  abline(v=q95_beta_Cell, col="red")

# take a second look at Sale, Cleanse, Pastured, Temp
  # Sale: possibly negative effect
  # Cleanse: possibly negative effect
  # Pastured: possibly negative effect
  # Temp: possibly weak positive effect

# calculate 99% and 99.9% posterior intervals
q99_beta_Sale <- quantile(beta_Sale, probs = c(0.01, 0.99))
q99_beta_Cleanse <- quantile(beta_Cleanse, probs = c(0.01, 0.99))
q99_beta_Pastured <- quantile(beta_Pastured, probs = c(0.01, 0.99))
q99_beta_Temp <- quantile(beta_Temp, probs = c(0.01, 0.99))

q999_beta_Sale <- quantile(beta_Sale, probs = c(0.001, 0.999))
q999_beta_Cleanse <- quantile(beta_Cleanse, probs = c(0.001, 0.999))
q999_beta_Pastured <- quantile(beta_Pastured, probs = c(0.001, 0.999))
q999_beta_Temp <- quantile(beta_Temp, probs = c(0.001, 0.999))

par(mfrow=c(2,2))
  
plot(density(beta_Sale), main="Posterior of beta: Sale")
  abline(v=q95_beta_Sale, col="red")
  abline(v=q99_beta_Sale, col="blue")
  abline(v=q999_beta_Sale, col="blue") 
  
plot(density(beta_Cleanse), main="Posterior of beta: Cleanse")
  abline(v=q95_beta_Cleanse, col="red")
  abline(v=q99_beta_Cleanse, col="blue")
  abline(v=q999_beta_Cleanse, col="blue")
  
plot(density(beta_Pastured), main="Posterior of beta: Pastured")
  abline(v=q95_beta_Pastured, col="red")
  abline(v=q99_beta_Pastured, col="blue")
  abline(v=q999_beta_Pastured, col="blue")
  
plot(density(beta_Temp), main="Posterior of beta: Temp")
  abline(v=q95_beta_Temp, col="red")
  abline(v=q99_beta_Temp, col="blue")
  abline(v=q999_beta_Temp, col="blue")

# probabilities for an opposite effect  
sum(beta_Sale > 0)/length(beta_Sale)
sum(beta_Cleanse > 0)/length(beta_Cleanse)
sum(beta_Pastured > 0)/length(beta_Pastured)
sum(beta_Temp < 0)/length(beta_Temp)

# store results in a data frame
qlower <- c(quantile(beta_Sale, probs = 0.05),
            quantile(beta_Cleanse, probs = 0.05),
            quantile(beta_Feed, probs = 0.05),
            quantile(beta_Pastured, probs = 0.05),
            quantile(beta_Maint, probs = 0.05),
            quantile(beta_Rain, probs = 0.05),
            quantile(beta_Temp, probs = 0.05),
            quantile(beta_Bact, probs = 0.05),
            quantile(beta_Cell, probs = 0.05))

qupper <- c(quantile(beta_Sale, probs = 0.95),
            quantile(beta_Cleanse, probs = 0.95),
            quantile(beta_Feed, probs = 0.95),
            quantile(beta_Pastured, probs = 0.95),
            quantile(beta_Maint, probs = 0.95),
            quantile(beta_Rain, probs = 0.95),
            quantile(beta_Temp, probs = 0.95),
            quantile(beta_Bact, probs = 0.95),
            quantile(beta_Cell, probs = 0.95))

pabove0 <- c(sum(beta_Sale > 0)/length(beta_Sale),
             sum(beta_Cleanse > 0)/length(beta_Cleanse),
             sum(beta_Feed > 0)/length(beta_Feed),
             sum(beta_Pastured > 0)/length(beta_Pastured),
             sum(beta_Maint > 0)/length(beta_Maint),
             sum(beta_Rain > 0)/length(beta_Rain),
             sum(beta_Temp > 0)/length(beta_Temp),
             sum(beta_Bact > 0)/length(beta_Bact),
             sum(beta_Cell > 0)/length(beta_Cell))

results <- data.frame(qlower, qupper, pabove0)
rownames(results) <- c("Sale", "Cleanse", "Feed", "Pastured", "Maint", "Rain", "Temp", "Bact", "Cell")
colnames(results) <- c("5%", "95%", "p(beta>0)")
results

# 5%        95% p(beta>0)
# Sale     -62.6330909 -3.6556856    0.0003
# Cleanse   -4.9258859 -0.6662644    0.0052
# Feed      -0.5840992  0.9536535    0.6765
# Pastured  -1.9640363 -0.3202455    0.0103
# Maint     -2.7578217  0.0692628    0.0632
# Rain      -0.1950234  0.2017243    0.5197
# Temp       0.4933904  1.2546634    0.9998
# Bact      -0.1261676  0.2564108    0.7832
# Cell      -0.3366392  0.2394072    0.4096


# citations and versions

citation()
# R Core Team (2018). R: A language and environment for statistical computing. R Foundation for Statistical Computing, Vienna, Austria. URL https://www.R-project.org/.

R.Version()
# $version.string
# [1] "R version 3.4.4 (2018-03-15)"

citation("rjags")
# Martyn Plummer (2016). rjags: Bayesian Graphical Models using MCMC. R package version 4-6. https://CRAN.R-project.org/package=rjags

# JAGS software version 4.3.0 (64-bit)

